/*###########################################################################
#                                                                           #
#                                BMP2RAW                                    #
#                                                                           #
###########################################################################*/
/*

Released in the public domain


Converts 128x64x1 BMP picture to raw binary file.
the pixels in the resulting file are stored top-to-bottom and left-to-right.
The MSB of each byte is a "left-pixel", the MSB is a "right-pixel".
Works on 16, 32 and 64-bit CPU, little and big-endian.

october 1st, 2002 initial release by David TAILLANDIER

Tested with pictures from Microsoft PaintBrush and ACDSee.


The BMP format for 128x64x1 pictures
------------------------------------

header: 14 bytes
     signature        2 bytes      'BM' 
     file size        4 bytes      in byte (1086 for 128x64x1)
     reserved         4 bytes      (0,0,0,0) 
     data offset      4 bytes      (62 for 128x64x1)

Info header: 40 bytes
     size of header   4 bytes      (40)
     width            4 bytes      in pixel (128 for 128x64x1) 
     height           4 bytes      in pixel (64 for 128x64x1) 
     planes           2 bytes      ? (1) 
     bit per pixel    2 bytes      (1 for 128x64x1)
     compression type 4 bytes      (0 for 128x64x1)
     compressed size  4 bytes      in byte (0 or 1024 for 128x64x1)
     Xpixel per meter 4 bytes      ignore this
     Ypixel per meter 4 bytes      ignore this
     colors used      4 bytes      (0 for 128x64x1 (?!))
     important colors 4 bytes      (0 for 128x64x1)

color table: 4 x number of colors  (8 bytes for 1024x64x1)
     reb              1 byte       x
     green            1 byte       x
     blue             1 byte       x
     reserved         1 byte       0

datas: (1024 bytes for 128x64x1)
     stored bottom-to-top and left-to-right (upside-down)

*/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#define BMP_SIZE     1086
#define RAW_SIZE     1024
#define BMP_HEADER_SIZE   (14+40+2*4)

/*############################# error_exit ################################*/
void
error_exit( const char * text, ... )
{
     va_list vargs;

     printf( "ERROR: " );

     va_start( vargs, text );
     vprintf( text, vargs );
     va_end( vargs );

     printf( "\n" );

     exit( -1 );
}

/*############################## usage ####################################*/
static
void
usage( void )
{
     error_exit( "   usage is:     bmp2raw <source-file> <dest-file>\n"
                 "\n"
                 "This program converts BMP files (monochrome) into raw format."
                 "See program source for more explainations.\n"
               );
}

/*################################ read_file ##############################*/
static
void
read_file( const char * filename, char * buffer, unsigned int size )
{
     FILE * f;
     struct stat s;

     if ( stat(filename,&s) != 0 )
          error_exit( "unable to detect the length of [%s]. Maybe the file does not exist", filename );
     if ( (unsigned int)(s.st_size) != size )
          error_exit( "the file have an incorrect size [%s]. %i instead of %i bytes", filename, s.st_size, size );

     f = fopen( filename ,"rb" );
     if ( f == NULL )
          error_exit( "unable to open input file" );

     if ( fread(buffer,1,size,f) != size )
          error_exit( "read error on [%s]", filename );

     if ( fclose(f) != 0 )
          error_exit( "close error on [%s]", filename );
}

/*################################ write_file #############################*/
static
void
write_file( const char * filename, char * buffer, unsigned int size )
{
     FILE * f;

     f = fopen( filename, "wb" );
     if ( f == NULL )
          error_exit( "unable to create file [%s]", filename );

     if ( fwrite(buffer,1,size,f) != size )
          error_exit( "write error on [%s]", filename );

     if ( fclose(f) != 0 )
          error_exit( "close error on [%s]", filename );
}

/*########################### check_bmp ###################################*/
static
void
check_bmp( const char * bmp )
{
     char check[] = { 0x42, 0x4d,                 /* BM */
                      0x3e, 0x04, 0x00, 0x00,     /* 1086 */
                      0x00, 0x00, 0x00, 0x00,     /* reserved (=0) */
                      0x3e, 0x00, 0x00, 0x00,     /* data offset (62) */
                      0x28, 0x00, 0x00, 0x00,     /* size of header (40) */
                      0x80, 0x00, 0x00, 0x00,     /* width (128) */
                      0x40, 0x00, 0x00, 0x00,     /* height (64) */
                      0x01, 0x00,                 /* planes (1) */
                      0x01, 0x00,                 /* bit per pixel (1) */
                      0x00, 0x00, 0x00, 0x00      /* compression type (0) */
                    };

     if ( memcmp(check,bmp,sizeof check) != 0 )
          error_exit( "the header seems not from a 128x64x1 BMP file" );
}

/*############################# convert ###################################*/
static
void
convert( const char * bmp, char * raw )
{
     int i;

     for( i=0; i!=64; i++ )
          memcpy( raw+(16*i), bmp+1024-16-(16*i), 16 );

     for ( i=0; i!=1024; i++ )
          raw[i] = ~raw[i];
}

/*############################ main #######################################*/
int
main( int argc, char *argv[] )
{
     char * bmp_data;
     char * raw_data;

     if( argc != 3 )
       usage();

     bmp_data = malloc( BMP_SIZE );
     raw_data = malloc( RAW_SIZE );
     if ( bmp_data==NULL || raw_data==NULL )
          error_exit( "not enought memory (this should NEVER arise !)" );

     read_file( argv[1], bmp_data, BMP_SIZE );
     check_bmp( bmp_data );
     convert( bmp_data + BMP_HEADER_SIZE, raw_data );
     write_file( argv[2], raw_data, RAW_SIZE );

     printf( "Done." );
     return 0;
}

/*############################# END OF FILE ###############################*/
