/*

Released in the public domain


History:

february 9th, 2003   David TAILLANDIER     Initial release

*/

#include <stdio.h>
#include <time.h>

#include "lcd_lib.h"

static int lib_init_status = 0;      /* 0 = init not done */
                                     /* 1 = init done     */
static int loc_loop_count = 0;

/* because every compilers and OSes have a different way to write
 * a byte to a port, we need to define a macro
 */
#if defined __DJGPP__
#     include <pc.h>
#     define outportb(address,byte)    outportb((address),(byte))
#endif

#if defined __minix
      void outb(unsigned short port, unsigned char value);
#     define outportb(address,byte)    outb((address),(byte))
#endif

/* NetBSD (and OpenBSD ?) */
#if defined __NetBSD__
#    include <machine/pio.h>
#    include <sys/types.h>
#    include <machine/sysarch.h>
#    define outportb(address,byte)  outb((address),(byte))
#endif

#if defined __CYGWIN__
#define outportb(address,byte)   ({ __asm__ __volatile__ ("outb %b0,%w1"::"a" (byte), "d" (address)); })
#endif

/*############################## lcd_wait #################################*/
static
void
lcd_wait( void )
{
     int i;

     for (i=0; i!=loc_loop_count; i++)
          ;
}

/*############################# lcd_lib_init ##############################*/
/* If 'loop_count'==-1 then the value does not change
 * If 'loop_count'==0 then the value defined in the macro LCD_DELAY is used
 * return value:  0 = error   (false)
 *                1 = ok      (true)
 */
int
lcd_lib_init( int loop_count )
{
     if (lib_init_status != 0)
          return 0;     /* already done */

     if (loop_count == 0)
          loop_count = LCD_DELAY;
     if (loop_count == -1)
          loop_count = loc_loop_count;
     loc_loop_count = loop_count;

/* some tricks to gain access to outportb() function */
#if defined __minix
  {
     FILE * file;

     file = fopen( "/dev/mem", "rb");
     if (file == NULL)
          return 0;
     if (fclose(file) != 0)
          return 0;
  }
#endif

#if defined __NetBSD__
  {
     unsigned long iomap[32];

     if (i386_get_ioperm(iomap) == -1)
          return 0;

     /* the bit must be cleared to gain access to corresponding port */
     iomap[LCD_PORT_DATA   /32] &= ~( 1 << (LCD_PORT_DATA   %32) );
     iomap[LCD_PORT_CONTROL/32] &= ~( 1 << (LCD_PORT_CONTROL%32) );

     if (i386_set_ioperm(iomap) == -1)
          return 0;
  }
#endif

     /* the parallel port can be in a random state. We have to drop E, but
      * doing this let the LCD to accept a command or a data. We then drop E
      * without selecting any chip. As we need a small delay between 'no-chip
      * -selection' and the drop of E, we have to first raise E and deselect
      * the chips.
      */
     outportb(LCD_PORT_CONTROL, LCD_E_PIN^PPORT_XOR);
     lcd_wait();
     outportb(LCD_PORT_CONTROL, 0^PPORT_XOR);
     lcd_wait();

     /* we don't erase the LCD screen nor we switch it on or off because this
      * is not desirable in a number of cases. This is the job of the
      * user-program.
      */

     lib_init_status = 1;

     return 1;
}

/*############################# lcd_lib_end ###############################*/
/* return value:  0 = error   (false)
 *                1 = ok      (true)
 */
int
lcd_lib_end( void )
{
     if ( lib_init_status != 1 )
          return 0;

     lib_init_status = 0;

     return 1;
}

/*############################# send_byte #################################*/
/* send a data to LCD screen or to LCD controler
 * chip = 0 or 1   (left or right)
 * dest = 0   --> to controler (value = command to send)
 * dest = 1   --> to screen (value = 8 vertical pixels)
 */
static
void
send_byte( int chip, int dest, int value )
{
     unsigned char command;

     if (chip == 0)     command = LCD_CS1_PIN ^ PPORT_XOR;
     else                command = LCD_CS2_PIN ^ PPORT_XOR;

     if (dest == 1)
          command ^= LCD_DI_PIN;     /* 0+LCD_CSx_PIN for controler      LCD_DI_PIN+LCD_CSx_PIN for screen */

     outportb(LCD_PORT_DATA, value);
     outportb(LCD_PORT_CONTROL, LCD_E_PIN^command);                  /* rise ENABLE pin  */
     lcd_wait();
     outportb(LCD_PORT_CONTROL, command);                            /* lower ENABLE pin */
     lcd_wait();

/* note about binary logic:
 * The value sent to the control port must be:    (CS1|DI|E)^PPORT_XOR  for example
 *                                                or (CS2| 0|E)^PPORT_XOR
 *                                                or etc etc
 * rule 1 - The bits of CS1 and CS2 and DI and E never overlap between them. Then CS1|DI|E = CS1^DI^E
 * rule 2 - The XOR operation can be swaped as we want: A^B^C = C^A^B = B^A^C
 *
 * rules 1 and 2 give us: (CS1|DI|E)^PPORT_XOR = CS1 ^ PPORT_XOR ^ DI ^ E
 */
}

/*############################ lcd_on_off #################################*/
/*  on_off==0 to turn off the LCD-screen
 *  on_off!=0 to turn on  the LCD-screen
 */
void
lcd_on_off( int on_off )
{
     if( lib_init_status == 0 )
          return;

     if ( on_off != 0 )
          on_off = 1;

     send_byte( 0, 0, 0x3E + on_off );
     send_byte( 1, 0, 0x3E + on_off );
}

/*########################### lcd_start_line ##############################*/
/*
static
void
lcd_start_line( int chip, int line )
{*/

#define lcd_start_line( chip, line )    send_byte( (chip), 0,  0xC0+(line) )

/*}*/

/*############################ lcd_set_page ###############################*/
/*static
void
lcd_set_page( int chip, int page )
{*/

#define lcd_set_page( chip, page )    send_byte( (chip), 0,  0xB8+(page) )

/*}*/

/*############################ lcd_set_address ############################*/
/*static
void
lcd_set_address( int chip, int address )
{*/

#define lcd_set_address( chip, address )      send_byte( (chip), 0, 0x40+(address) )

/*}*/

/*############################# lcd_fill ##################################*/
void
lcd_fill( unsigned char value )
{
     int address;
     int page;
     int chip;

     if( lib_init_status == 0 )
          return;

     for ( chip=0; chip!=2; chip++ )
       {
          lcd_set_address( chip, 0 );
          lcd_start_line( chip, 0 );

          for ( page=0; page!=8; page++ )
            {
               lcd_set_page( chip, page );
               for ( address=0; address!=64; address++ )
                    send_byte( chip, 1, value );
            }
       }
}

#if LCD_ORIENTATION == 0  /* up */
/*############################# lcd_img ###################################*/
void
lcd_img( const unsigned char * data_ptr )
{
     int address;
     int page;
     int chip;
     unsigned char byte;

     if( lib_init_status == 0 )
          return;

     lcd_set_address( 0, 0 );       lcd_start_line( 0, 0 );
     lcd_set_address( 1, 0 );       lcd_start_line( 1, 0 );

     for ( page=0; page!=8; page++, data_ptr+=(128-16) )
       {
          for ( chip=0; chip!=2; chip++ )
            {
               lcd_set_page( chip, page );
               for ( address=0 ; address!=64; address++ )
                 {
                    byte =   (      (   ( (*(data_ptr+  0)) >> (7-(address%8)) )   &1   )     << 0      )
                           + (      (   ( (*(data_ptr+ 16)) >> (7-(address%8)) )   &1   )     << 1      )
                           + (      (   ( (*(data_ptr+ 32)) >> (7-(address%8)) )   &1   )     << 2      )
                           + (      (   ( (*(data_ptr+ 48)) >> (7-(address%8)) )   &1   )     << 3      )
                           + (      (   ( (*(data_ptr+ 64)) >> (7-(address%8)) )   &1   )     << 4      )
                           + (      (   ( (*(data_ptr+ 80)) >> (7-(address%8)) )   &1   )     << 5      )
                           + (      (   ( (*(data_ptr+ 96)) >> (7-(address%8)) )   &1   )     << 6      )
                           + (      (   ( (*(data_ptr+112)) >> (7-(address%8)) )   &1   )     << 7      );
                    send_byte( chip, 1, byte );
                    if ( address%8 == 7 )
                         data_ptr++;
                 }   /* for(address) */
            }   /* for(chip) */
       }   /* for(page) */
}
#endif

#if LCD_ORIENTATION == 1  /* down */
/*############################# lcd_img ###################################*/
void
lcd_img( const unsigned char * data_ptr )
{
     int address;
     int page;
     int chip;
     unsigned char byte;

     if( lib_init_status == 0 )
          return;

     lcd_set_address( 0, 0 );       lcd_start_line( 0, 0 );
     lcd_set_address( 1, 0 );       lcd_start_line( 1, 0 );

     data_ptr += 1023;              /* start at the end of the picture */

     for ( page=0; page!=8; page++, data_ptr-=(128-16) )
       {
          for ( chip=0; chip!=2; chip++ )
            {
               lcd_set_page( chip, page );
               for ( address=0 ; address!=64; address++ )
                 {
                    byte =   (      (   ( (*(data_ptr-  0)) >>    (address%8) )   &1    )     << 0      )
                           + (      (   ( (*(data_ptr- 16)) >>    (address%8) )   &1    )     << 1      )
                           + (      (   ( (*(data_ptr- 32)) >>    (address%8) )   &1    )     << 2      )
                           + (      (   ( (*(data_ptr- 48)) >>    (address%8) )   &1    )     << 3      )
                           + (      (   ( (*(data_ptr- 64)) >>    (address%8) )   &1    )     << 4      )
                           + (      (   ( (*(data_ptr- 80)) >>    (address%8) )   &1    )     << 5      )
                           + (      (   ( (*(data_ptr- 96)) >>    (address%8) )   &1    )     << 6      )
                           + (      (   ( (*(data_ptr-112)) >>    (address%8) )   &1    )     << 7      );
                    send_byte( chip, 1, byte );
                    if ( address%8 == 7 )
                         data_ptr--;
                 }   /* for(address) */
            }   /* for(chip) */
       }   /* for(page) */
}
#endif

/*########################### END OF FILE #################################*/
