/*

Released in the public domain



History:

february 9th, 2003   David TAILLANDIER     Initial release

*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#if defined __CYGWIN__
#     include <unistd.h>   /* for sleep() */
#endif

#include "lcd_lib_conf.h"


/* because every compilers and OSes have a different way to write
 * a byte to a port, we need to define a macro
 */
#if defined __DJGPP__
#     include <pc.h>
#     define outportb(address,byte)    outportb((address),(byte))
#endif

/* NetBSD (and OpenBSD ?) */
#if defined __NetBSD__
#    include <machine/pio.h>
#    include <sys/types.h>
#    include <machine/sysarch.h>
#    define outportb(address,byte)  outb((address),(byte))
#endif

#if defined __CYGWIN__
#define outportb(address,byte)   ({ __asm__ __volatile__ ("outb %b0,%w1"::"a" (byte), "d" (address)); })
#endif


/*############################## usage ####################################*/
static
void
usage( void )
{
     printf( "lcd_parallel_port_test   v0.9\n"
             "This program help to check if an LCD module is correctly wired\n"
             "to the parallel port of the computer.\n"
             "\n"
             "Command line is:   lcd_parallel_port_test <lpt_number> <signal>\n"
             "\n"
             "   <base-address> is the parallel-port number as in ms-dos\n"
             "                    LPT1 (0x378)      LPT2 (0x278)     LPT3 (0x3BC)\n"
             "\n"
             "   <signal> is the name of the signal to test\n"
             "                    E   for the 'E' pin\n"
             "                    DI  for the 'DI' pin\n"
             "                    CS1 for the 'CS1' pin\n"
             "                    CS2 for the 'CS2' pin\n"
             "                    D1  for the first data pin\n"
             "                    D2  for the second data pin\n"
             "                    D3...D8 for the other data pins\n"
             "\n"
             "The selected pin is set to 'high' during 1 second, the set to 'low'\n"
             "during 3 seconds, then 'high' during 1 second, then 'low'...\n"
             "This allow to easely test with a voltmeter.\n"
             "\n"
             "note: pin numbers are set in accordance with lcd_lib_conf.h\n"
           );
}

/*############################## error_exit ###############################*/
static
void
error_exit( const char * text, ... )
{
     va_list vargs;

     printf( "ERROR: " );

     va_start( vargs, text );
     vprintf( text, vargs );
     va_end( vargs );

     printf( "\n" );

     exit( -1 );
}

/*############################## main #####################################*/
int
main( int argc, char * argv[] )
{
     struct {
          char * name;
          int    address;
     } port[] = {   { "LPT1", 0x378 },    { "LPT2", 0x278 },    { "LPT3", 0x3bc }   };

     struct {
          char *        name;
          int           type;        /* 0=data-pin   1=control-pin */
          unsigned char value;
     } pin[] = {   { "E",   1, LCD_E_PIN   },      { "DI",  1, LCD_DI_PIN  },
                   { "CS1", 1, LCD_CS1_PIN },      { "CS2", 1, LCD_CS2_PIN },
                   { "D1",  0,   1         },      { "D2",  0,   2         },
                   { "D3",  0,   4         },      { "D4",  0,   8         },
                   { "D5",  0,  16         },      { "D6",  0,  32         },
                   { "D7",  0,  64         },      { "D8",  0, 128         }    };

     int address = -1;
     int value = -1;
     int mask = 0;
     int i;

     if ( argc != 3 )
          usage();

     for ( i=0; port[i].name[0]!=0; i++ )
       {
          if ( stricmp(port[i].name,argv[1]) == 0 )
            {
               address = port[i].address;
               break;
            }
       }
     if ( address == -1 )
          error_exit( "The first parameter must be LPT1 or LPT2 or LPT3" );

     for ( i=0; pin[i].name[0]!=0; i++ )
       {
          if ( stricmp(pin[i].name,argv[2]) == 0 )
            {
               value = pin[i].value;
               if ( pin[i].type == 1 )
                 {
                    address += 2;
                    mask = PPORT_LOGIC;
                 }
               break;
            }
       }
     if ( value == -1 )
          error_exit( "The second parameter must be E, DI, CS1, CS2, D1, D2, D3, D4, D5, D6, D7 or D8" );

     printf( "\nport=%3X  value=%2X\n", address, value );

     for (;;)
       {
          outportb( address, value^mask );
          sleep( 1 );
          outportb( address, 0^mask );
          sleep( 3 );
       }

     return 0;        /* never reached */
}

/*########################### END OF FILE #################################*/
